import {sortByNumberAscending, sortByStringAscending, sortByDateAscending} from '../../services/sorting.service';
import {Priority, ScrumTask} from '../../services/backend.service';

export abstract class TableComponentBase<T> {
  public sortBy: string;
  public sortDescending = false;
  public items: T[] = [];

  protected doNumericSort(by: string, key: (item: T) => number) {
    if (this.sortBy === by) {
      this.sortDescending = !this.sortDescending;
    } else {
      this.sortBy = by;
    }

    this.items = sortByNumberAscending(this.items, key);
    if (this.sortDescending) {
      this.items = this.items.reverse();
    }
  }

  protected doStringSort(by: string, key: (item: T) => string) {
    if (this.sortBy === by) {
      this.sortDescending = !this.sortDescending;
    } else {
      this.sortBy = by;
    }

    this.items = sortByStringAscending(this.items, key);
    if (this.sortDescending) {
      this.items = this.items.reverse();
    }
  }

  protected doDateSort(by: string, key: (item: T) => Date) {
    if (this.sortBy === by) {
      this.sortDescending = !this.sortDescending;
    } else {
      this.sortBy = by;
    }

    this.items = sortByDateAscending(this.items, key);
    if (this.sortDescending) {
      this.items = this.items.reverse();
    }
  }

  public getAllPriorities(): string[] {
    return Object.values(Priority);
  }

  public sortById() {
    this.doNumericSort('id', (obj) => obj.id);
  }

  public sortByTitle() {
    this.doStringSort('title', (obj) => obj.title);
  }
}
